unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, TDScene, TDComplex, TDCamera, TDStarField,
  ActnList, TDLight, ComCtrls, TDLines;

type
  TThruster=record
    AForward, ABackward, ALeft, ARight, AUp, ADown: Boolean;
    RLeft, RRight, RUp, RDown, RRollLeft, RRollRight: Boolean;
  end;

  TForm1 = class(TForm)
    Timer50: TTimer;
    ActionList1: TActionList;
    ActionToggleRenderMode: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Timer50Timer(Sender: TObject);
    procedure ActionToggleRenderModeExecute(Sender: TObject);
  private
    Buffer: TBitmap;
    ATime: Integer;
    ShowFPS: Boolean;
    Thruster: TThruster;
    rx, ry, rz, vx, vy, vz: Longint;
  public
    Scene: TTDScene;
    Cube: TTDComplex;
    Plane: TTDLines;
    Cube2, Cube3, Cube4, Cube5, CubeL: TTDComplex;
    Camera: TTDCamera;
    Starfield: TTDStarField;
    Light, Light2: TTDLight;
    procedure ApplicationIdle(Sender: TObject; var Done: Boolean);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
  Vertexes: array [0..3] of Integer;
begin
  Buffer := TBitmap.Create;
  Buffer.Width := ClientWidth;
  Buffer.Height := ClientHeight;

  Scene:=TTDScene.Create;
  Starfield:=TTDStarField.Create(Scene);

  Camera:=TTDCamera.Create(Scene);
  Cube2:=TTDComplex.Create(Scene);
  Cube:=TTDComplex.Create(Scene);
  Cube.LoadObject(ExtractFilePath(Application.ExeName)+'worcem.tdo','Worcem');
  Light:=TTDLight.Create(Scene);
  Light.SetColor(clWhite);
  Light.Move(90,50,200);
  Starfield.Init(1024);
  ATime := 0;
  ShowFPS:=True;
  Camera.MoveRelative(0,0,-50);

  Cube2.AddVertex(-50000000,-50000000,-50000000);
  Cube2.AddVertex(-50000000,+50000000,-50000000);
  Cube2.AddVertex(+50000000,+50000000,-50000000);
  Cube2.AddVertex(+50000000,-50000000,-50000000);
  Cube2.AddVertex(-50000000,-50000000,+50000000);
  Cube2.AddVertex(-50000000,+50000000,+50000000);
  Cube2.AddVertex(+50000000,+50000000,+50000000);
  Cube2.AddVertex(+50000000,-50000000,+50000000);
  Cube2.Move(0,0,5000000000.0*1000);
  Vertexes[0]:=0;
  Vertexes[1]:=1;
  Vertexes[2]:=2;
  Vertexes[3]:=3;
  Cube2.AddPlane(Vertexes,clWhite,clWhite);
  Vertexes[0]:=7;
  Vertexes[1]:=6;
  Vertexes[2]:=5;
  Vertexes[3]:=4;
  Cube2.AddPlane(Vertexes,clWhite,clWhite);
  Vertexes[0]:=3;
  Vertexes[1]:=2;
  Vertexes[2]:=6;
  Vertexes[3]:=7;
  Cube2.AddPlane(Vertexes,clWhite,clWhite);
  Vertexes[0]:=4;
  Vertexes[1]:=5;
  Vertexes[2]:=1;
  Vertexes[3]:=0;
  Cube2.AddPlane(Vertexes,clWhite,clWhite);
  Vertexes[0]:=1;
  Vertexes[1]:=5;
  Vertexes[2]:=6;
  Vertexes[3]:=2;
  Cube2.AddPlane(Vertexes,clWhite,clWhite);
  Vertexes[0]:=4;
  Vertexes[1]:=0;
  Vertexes[2]:=3;
  Vertexes[3]:=7;
  Cube2.AddPlane(Vertexes,clWhite,clWhite);
  Cube.Move(0,0,10);
  Cube.Rotate(0,180*Pi/180,0);
  Application.OnIdle:= ApplicationIdle;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Scene.Free;
  Buffer.Free;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
  Ord('W'): Thruster.AForward:=True;
  Ord('S'): Thruster.ABackward:=True;
  VK_RIGHT: Thruster.ALeft:=True;
  VK_LEFT : Thruster.ARight:=True;
  VK_DOWN : Thruster.AUp:=True;
  VK_UP   : Thruster.ADown:=True;
  Ord('J'): Thruster.RLeft:=True;
  Ord('L'): Thruster.RRight:=True;
  Ord('U'): Thruster.RRollLeft:=True;
  Ord('O'): Thruster.RRollRight:=True;
  Ord('I'): Thruster.RUp:=True;
  Ord('K'): Thruster.RDown:=True;
  Ord('G'): Light.Move(0,0,5);
  Ord('B'): Light.Move(0,0,-5);
  Ord('V'): Light.Move(-5,0,0);
  Ord('N'): Light.Move(5,0,0);
  end;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
  Ord('W'): Thruster.AForward:=False;
  Ord('S'): Thruster.ABackward:=False;
  VK_RIGHT: Thruster.ALeft:=False;
  VK_LEFT : Thruster.ARight:=False;
  VK_DOWN : Thruster.AUp:=False;
  VK_UP   : Thruster.ADown:=False;
  Ord('J'): Thruster.RLeft:=False;
  Ord('L'): Thruster.RRight:=False;
  Ord('U'): Thruster.RRollLeft:=False;
  Ord('O'): Thruster.RRollRight:=False;
  Ord('I'): Thruster.RUp:=False;
  Ord('K'): Thruster.RDown:=False;
  end;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  Light.Move(0,0,-10);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Light.Move(0,0,-10);
end;

procedure TForm1.ApplicationIdle(Sender: TObject; var Done: Boolean);
var
  BTime: Integer;
begin
  BTime := GetCurrentTime;
  if Scene.Changed then
   begin
     PatBlt( Buffer.Canvas.Handle, 0, 0, Buffer.Width, Buffer.Height, BLACKNESS );

     if ShowFPS then
      with Buffer.Canvas do
       begin
         Font.Color := clWhite;
         Brush.Color := clBlack;
         TextOut( 5, 5, Format( '%d Frames/s',[ 1000 div (BTime-ATime) ] ) );
       end;
     ATime := BTime;

     Scene.Render(Buffer.Canvas,Camera);
     BitBlt( Canvas.Handle, 0, 0, ClientWidth, ClientHeight, Buffer.Canvas.Handle, 0, 0, SRCCOPY );

     Scene.Changed :=False
   end;
  Application.ProcessMessages;
  Done := False;
end;

procedure TForm1.Timer50Timer(Sender: TObject);
const
  rc=1;
  d=1;
begin
  if Thruster.ABackward then vz:=vz-5;
  if Thruster.AForward then vz:=vz+5;
  if not Thruster.AForward and not Thruster.ABackward then
   if vz>0 then vz:=vz-1 else if vz<0 then vz:=vz+1;
  if Thruster.ALeft then vx:=vx-5;
  if Thruster.ARight then vx:=vx+5;
  if not Thruster.ALeft and not Thruster.ARight then
   if vx>0 then vx:=vx-1 else if vx<0 then vx:=vx+1;
  if Thruster.ADown then vy:=vy-5;
  if Thruster.AUp then vy:=vy+5;
  if not Thruster.AUp and not Thruster.ADown then
    if vy>0 then vy:=vy-1 else if vy<0 then vy:=vy+1;
  if Thruster.RLeft then ry:=ry-rc;
  if Thruster.RRight then ry:=ry+rc;
  if not Thruster.RLeft and not Thruster.RRight then
    if ry>0 then
      ry:=ry-rc div d
    else if ry<0 then
      ry:=ry+rc div d;
  if Thruster.RRollRight then rz:=rz-rc;
  if Thruster.RRollLeft then rz:=rz+rc;
  if not Thruster.RRollLeft and not Thruster.RRollRight then
    if rz>0 then rz:=rz-rc div d else if rz<0 then rz:=rz+rc div d;
  if Thruster.RDown then rx:=rx-rc;
  if Thruster.Rup then rx:=rx+rc;
  if not Thruster.RUp and not Thruster.RDown then
    if rx>0 then rx:=rx-rc div d else if rx<0 then rx:=rx+rc div d;
  if (rx<>0) or (ry<>0) or (rz<>0) then
  Camera.RotateRelative(rx*Pi/1800,ry*Pi/1800,rz*Pi/1800);
  if (vx<>0) or (vy<>0) or (vz<>0) then
  Camera.MoveRelative(vx/20,vy/20,vz/20);
end;

procedure TForm1.ActionToggleRenderModeExecute(Sender: TObject);
begin
  case Camera.RenderMode of
  rmVertexes: Camera.RenderMode := rmLines;
  rmLines   : Camera.RenderMode := rmPlanes;
  rmPlanes  : Camera.RenderMode := rmVertexes;
  end;
end;

end.





